/*
* Copyright (c) 2017 Fraunhofer FOKUS.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the
* Eclipse Public License version 2.0 which accompanies
* this distribution, and is available at
* https://opensource.org/licenses/EPL-2.0
*
* Contributors:
*
* Torben Jastrow (Fraunhofer FOKUS) 
* 
* **Initially developed in the context of DECIDE EU project www.DECIDE-h2020.eu */
package eu.DECIDEh2020.architect.plugin.editor;


import java.util.ArrayList;

import org.apache.jena.rdf.model.Resource;
import org.eclipse.core.resources.IMarker;
import org.eclipse.core.resources.IResourceChangeEvent;
import org.eclipse.core.resources.IResourceChangeListener;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.jface.dialogs.ErrorDialog;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Display;
import org.eclipse.ui.*;
import org.eclipse.ui.part.FileEditorInput;
import org.eclipse.ui.part.MultiPageEditorPart;
import org.eclipse.ui.part.MultiPageEditorSite;
import org.eclipse.ui.ide.IDE;
import org.eclipse.ui.views.contentoutline.*;
import org.eclipse.wst.json.core.contenttype.ContentTypeIdForJSON;
import org.eclipse.wst.sse.ui.StructuredTextEditor;

import com.google.gson.JsonSyntaxException;

import eu.DECIDEh2020.architect.appManager.persistence.JsonHandler;
import eu.DECIDEh2020.architect.appManager.persistence.JsonHandler.AppDescriptorWrapper;
import eu.DECIDEh2020.architect.appManager.models.AppDescriptor;
import eu.DECIDEh2020.architect.appManager.models.Microservice;
import eu.DECIDEh2020.architect.cloudpatterns.Patterns;
import eu.DECIDEh2020.architect.plugin.layoutComponents.PatternPageComposite;
import eu.DECIDEh2020.architect.plugin.layoutComponents.ProjectPageComposite;


/**
 * An example showing how to create a multi-page editor.
 * This example has 3 pages:
 * <ul>
 * <li>page 0 contains a nested text editor.
 * <li>page 1 allows you to change the font used in page 2
 * <li>page 2 shows the words in page 0 in sorted order
 * </ul>
 */
public class MultiPageEditor extends MultiPageEditorPart implements IResourceChangeListener{

	/** The text editor used in the json page. */
	private StructuredTextEditor editor;

	private AppDescriptor project;
	private Patterns patterns = Patterns.createDefaultPatterns();

	/** The composite used for the project page */
	private ProjectPageComposite ppage;
	
	private PatternPageComposite patPage;

	private IContentOutlinePage contentOutlinePage;
	/**
	 * Creates a multi-page editor example.
	 */
	public MultiPageEditor() {
		super();
		ResourcesPlugin.getWorkspace().addResourceChangeListener(this);
	}
	/**
	 * Creates the page of the multi-page editor,
	 * which contains a json editor.
	 */
	void createJSONPage() {
		try {
			editor = new StructuredTextEditor();
			int index = addPage(editor, getEditorInput());
			setPageText(index, editor.getTitle());


		} catch (PartInitException e) {
			ErrorDialog.openError(
					getSite().getShell(),
					"Error creating nested text editor",
					null,
					e.getStatus());
		}
		
	}
	/**
	 * Creates page 1 of the multi-page editor,
	 * which allows you to change the font used in page 2.
	 */
	void createProjectPage() {

		fromJSON();
		ppage = new ProjectPageComposite(getContainer(), project);
		int index = addPage(ppage);
		setPageText(index, "Project");
	}
	/**
	 * Creates page 2 of the multi-page editor,
	 * which shows the pattern recommender.
	 */
	void createPatternPage() {

		patPage = new PatternPageComposite(getContainer(), SWT.NONE,patterns);
		

		int index = addPage(patPage);
		setPageText(index, "Patterns");
	}
	/**
	 * Creates the pages of the multi-page editor.
	 */
	protected void createPages() {
		createJSONPage();
		createProjectPage();
		createPatternPage();
		setActivePage(1);
		refreshOutlineView();
	}
	/**
	 * The <code>MultiPageEditorPart</code> implementation of this 
	 * <code>IWorkbenchPart</code> method disposes all nested editors.
	 * Subclasses may extend.
	 */
	public void dispose() {
		ResourcesPlugin.getWorkspace().removeResourceChangeListener(this);
		super.dispose();
	}
	/**
	 * Saves the multi-page editor's document.
	 */
	public void doSave(IProgressMonitor monitor) {
		getEditor(0).doSave(monitor);
	}
	/**
	 * Saves the multi-page editor's document as another file.
	 * Also updates the text for page 0's tab, and updates this multi-page editor's input
	 * to correspond to the nested editor's.
	 */
	public void doSaveAs() {
		IEditorPart editor = getEditor(0);
		editor.doSaveAs();
		setPageText(0, editor.getTitle());
		setInput(editor.getEditorInput());
	}
	/* (non-Javadoc)
	 * Method declared on IEditorPart
	 */
	public void gotoMarker(IMarker marker) {
		setActivePage(0);
		IDE.gotoMarker(getEditor(0), marker);
	}
	/**
	 * The <code>MultiPageEditorExample</code> implementation of this method
	 * checks that the input is an instance of <code>IFileEditorInput</code>.
	 */
	public void init(IEditorSite site, IEditorInput editorInput)
			throws PartInitException {
		if (!(editorInput instanceof IFileEditorInput))
			throw new PartInitException("Invalid Input: Must be IFileEditorInput");
		super.init(site, editorInput);
	}
	/* (non-Javadoc)
	 * Method declared on IEditorPart.
	 */
	public boolean isSaveAsAllowed() {
		return true;
	}

	/**
	 * Calculates the contents of the then visible page when the it is activated.
	 */
	protected void pageChange(int newPageIndex) {
		super.pageChange(newPageIndex);

		switch (newPageIndex) {
		case 0:
			toJSON();
			break;
		case 1:
			fromJSON();
			ppage.refreshContent(project);
			break;
		case 2:
			toJSON();
			recomputeNFRs();
			
			break;
		}

	}
	/**
	 * Closes all project files on project close.
	 */
	public void resourceChanged(final IResourceChangeEvent event){
		if(event.getType() == IResourceChangeEvent.PRE_CLOSE){
			Display.getDefault().asyncExec(new Runnable(){
				public void run(){
					IWorkbenchPage[] pages = getSite().getWorkbenchWindow().getPages();
					for (int i = 0; i<pages.length; i++){
						if(((FileEditorInput)editor.getEditorInput()).getFile().getProject().equals(event.getResource())){
							IEditorPart editorPart = pages[i].findEditor(editor.getEditorInput());
							pages[i].closeEditor(editorPart,true);
						}
					}
				}            
			});
		}
	}
	
	void recomputeNFRs() {
		java.util.List<Resource> nfrs = new ArrayList<Resource>();;
		for(Microservice m: project.getMicroservices()) {
			nfrs.addAll(m.getNfrs());
		}
		patPage.updatePatterns(nfrs);
	}
	
	
	
	/**
	 * Converts JSON into a project Object
	 */
	void fromJSON() {
		
		
		
		String editorText = editor.getDocumentProvider().getDocument(editor.getEditorInput()).get();
		AppDescriptorWrapper app = null;
		try {
			app = JsonHandler.toAppDescriptor(editorText);
		} catch (JsonSyntaxException e) {
			// TODO Auto-generated catch block
			//e.printStackTrace();
			MessageDialog.openWarning(getSite().getShell(), "JSON file corrupt",e.getLocalizedMessage());
		
		}
		if (app != null) {
			
			if( app.isOldVersion()){
				MessageDialog.openWarning(getSite().getShell(), "Older version detected",
						"The application description file has a different Versionnumber than the one used in this version of "
						+ "the DECIDE plugin. This means, that some fields might be missing or that some pages of the editor "
						+ "might not work correctly. "
						+ "\nFile version is: " + app.getFileVersion() 
						+ "\nPlugin uses:     "+AppDescriptor.plugin_version +".");
			}
			
			
			project = app.getAppDescriptor();
		}
		if (project == null) {
			System.out.println("project is null after reading JSON");
		}
		
		
		
		
	}



	/**
	 * Converts a project Object into JSON
	 */
	void toJSON() {
		if (project != null) {

			// convert app descriptor object to json string
			String contents = JsonHandler.fromAppDescriptor(project);
			// get editor content
			String editorText = editor.getDocumentProvider().getDocument(editor.getEditorInput()).get();

			// if new json string is different from editor content, replace old with new
			if (contents != editorText)
				editor.getDocumentProvider().getDocument(editor.getEditorInput()).set(contents);

		}
	}
	
	
	@SuppressWarnings({ "rawtypes", "unchecked" })
	@Override
	public Object getAdapter(Class key) {
	    if (key.equals(IContentOutlinePage.class)) {
	        //return showOutlineView() ? getContentOutlinePage() : null;
	    	return getContentOutlinePage();
	    } else {
	        return super.getAdapter(key);
	    }
	}
	
	
	/**
	 * This accesses a cached version of the content outliner.
	 */
	public IContentOutlinePage getContentOutlinePage() {
	    if (contentOutlinePage == null) {
	        contentOutlinePage = (IContentOutlinePage) editor.getAdapter(IContentOutlinePage.class);
	        
	    }
	    return contentOutlinePage;
	}
	
	/**
	 * @see org.eclipse.ui.part.MultiPageEditorPart#createSite(org.eclipse.ui.IEditorPart)
	 */
	protected IEditorSite createSite(IEditorPart page) {
		IEditorSite site = null;
		if (page == editor) {
			site = new MultiPageEditorSite(this, page) {
				public String getId() {
					// Sets this ID so nested editor is configured for JSON source
					return ContentTypeIdForJSON.ContentTypeID_JSON + ".source"; //$NON-NLS-1$;
				}
			};
		}
		else {
			site = super.createSite(page);
		}
		return site;
	}
	/**
	 * Sorts the words in page 0, and shows them in page 2.
	 */
	void sortWords() {

		
	}

	public void refreshOutlineView() {
		//get the activePage
		IWorkbenchPage wp =  PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage();
		//Find desired view by its visual ID
		IViewPart myView=wp.findView("org.eclipse.ui.views.ContentOutline");

		//Hide the view :
		wp.hideView(myView);
		try {
			//show the view again     
			PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage().showView("org.eclipse.ui.views.ContentOutline");
		} catch (PartInitException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} 
	}
}
